#include "Click_8x8_G_config.h"
#include "Click_BarGraph_config.h"
#include "Click_EnOcean2_config.h"
#include "Click_MP3_config.h"
#include "Click_proxFusion_config.h"

#include "Click_8x8_G_types.h"
#include "Click_BarGraph_types.h"
#include "Click_EnOcean2_types.h"
#include "Click_MP3_types.h"
#include "Click_proxFusion_types.h"

#include "OLED_driver.h"
#include "Demo_Images.h"
#include "workStation_mp3.c"

sbit Mmc_Chip_Select           at PTC_PDOR.B3;

const uint8_t BUFFER_SIZE = 32;
const uint8_t MAX_SONGNAME_SIZE = 12;
const uint8_t MAX_SONGLIST_SIZE = 16;

////////////////////////////////////////////////////////////////////////// ENUM

enum T_touchStatus
{
    NEW_TOUCH,
    NOT_TOUCH
};
static enum T_touchStatus touchStatus;

enum T_buffedStatus
{
    BUFFER_UPLOADED,
    BUFFER_NOT_UPLOADED
};
static enum T_buffedStatus bufferStatus;

enum T_playStatus
{
     DEMO_PLAY,
     DEMO_STOP,
     DEMO_PAUSE
};
static enum T_playStatus playStatus;


enum T_songStatus
{
     DEMO_CURRENT_SONG,
     DEMO_NEXT_SONG,
     DEMO_PREV_SONG
};
static enum T_songStatus songStatus;


enum T_muteStatus
{
     DEMO_MUTE,
     DEMO_UNMUTE
};
static enum T_muteStatus muteStatus;


////////////////////////////////////////////////////////////////////// VARIABLE

static uint32_t fileSize;
static uint8_t fileBuffer[BUFFER_SIZE];

static uint8_t  dirContent[ MAX_SONGLIST_SIZE * 255 ];
static uint8_t  songList[ MAX_SONGLIST_SIZE ][ MAX_SONGNAME_SIZE ];
static uint16_t songSize;

static int8_t currentSongIndex = 0;
static uint8_t *currentSong;
static uint16_t currentVolume;
static uint8_t numMusic = 0;

// EnOcean buffers
uint8_t enocean_rxBuffer[ 512 ] = {0};
uint8_t enocean_dataBuffer[ 512 ] = {0};

// hexiwear touch
uint8_t tBuferCounter = 0;
uint8_t touchBuffer[5] = {0};

///////////////////////////////////////////////////////////// Static functions

static void api_incVolume();
static void api_decVolume();

static void api_muteVolume();
static void api_unmuteVolume();

static void api_playSong();
static void api_stopSong();

static void api_nextSong();
static void api_prevSong();

static void enocean2_packetHandler( enocean2_packet_t *packet );

//////////////////////////////////////////////////////////////////// TIMER INIT

void InitTimer0()
{
    SIM_SCGC6 |= (1 << PIT);
    NVIC_IntEnable(IVT_INT_PIT0);
    PIT_MCR = 0x00;
    PIT_LDVAL0 = 5999999;
    PIT_TCTRL0 |= 2;
    PIT_TCTRL0 |= 1;
}

void InitTimer1()
{
    SIM_SCGC6 |= (1 << PIT);
    NVIC_IntEnable(IVT_INT_PIT1);
    PIT_MCR = 0x00;
    PIT_LDVAL1 = 5999999;
    PIT_TCTRL1 |= 2;
    PIT_TCTRL1 |= 1;
}

////////////////////////////////////////////////////////////// STATIC FUNCTIONS

static void mmc_readDir(char ch)
{
    static uint16_t i;
    dirContent[i++] = ch;
}

static void api_getSongList()
{
    uint8_t *pStart;
    uint8_t *pNew;
    uint8_t ch;
    uint8_t counter = 0;
        
    pNew = dirContent;

#ifdef __SD_CARD__
    Mmc_Set_Interface(_MMC_INTERFACE_SDHC);
    while ( 0 != Mmc_Fat_Init());
    Mmc_Fat_Dir( mmc_readDir );
#endif
    while (1)
    {
        pStart = strstr((pNew), ".MP3");
        
        if (pStart != 0)
        {
            while (1)
            {
                ch = *(pStart - 1);
                if ( ch >= 33 && ch <= 122 )
                {
                    pStart--;
                    counter++;
                }
                else
                {
                    strncpy( songList[numMusic++], pStart, 4 + counter );
                    counter = 0;
                    break;
                }
            }
        }
        else
        {
           break;
        }
        pNew = ( pStart + MAX_SONGNAME_SIZE + 4);
    }
    numMusic --;
}

static void api_incVolume()
{
    uint8_t lVolume;
    uint8_t rVolume;
    uint8_t displayVolume;
   
    lVolume = (currentVolume >> 8 ) - 6;
    rVolume = (currentVolume & 0x00FF) - 6;
    currentVolume = ( lVolume << 8 ) | rVolume;
    
#ifdef __MP3_CLICK__
    mp3_cmdWrite(_MP3_VOL_ADDR, currentVolume);
#endif
#ifdef __BARGRAPH_CLICK__
    displayVolume = 10 - (rVolume / 6);
    bargraph_display(displayVolume);
#endif

}

static void api_decVolume()
{
    uint8_t lVolume;
    uint8_t rVolume;
    uint8_t displayVolume;

    lVolume = (currentVolume >> 8 ) + 6;
    rVolume = (currentVolume & 0x00FF) + 6;
    currentVolume = ( lVolume << 8 ) | rVolume;
    if (rVolume >= 0x3C)
    {
         rVolume = 0x3C;
         lVolume = 0x3C;
    }
    
#ifdef __MP3_CLICK__
    mp3_cmdWrite(_MP3_VOL_ADDR, currentVolume);
#endif
#ifdef __BARGRAPH_CLICK__
    displayVolume = 10 - (rVolume / 6);
    bargraph_display(displayVolume);
#endif
}

static void api_muteVolume()
{
    muteStatus = DEMO_MUTE;

#ifdef __MP3_CLICK__
   mp3_cmdWrite(_MP3_VOL_ADDR, 0x5F5F);
#endif
#ifdef __BARGRAPH_CLICK__
   bargraph_display(0);
#endif
#ifdef __8X8_CLICK__
   c8x8g_displayImage( MUTE_IMAGE );
#endif
}

static void api_unmuteVolume()
{
    muteStatus = DEMO_UNMUTE;
    
#ifdef __MP3_CLICK__
   mp3_cmdWrite(_MP3_VOL_ADDR, currentVolume);
#endif
#ifdef __BARGRAPH_CLICK__
   bargraph_display(10 - ((currentVolume >> 8) / 6));
#endif
#ifdef __8X8_CLICK__
   switch (playStatus)
   {
       case DEMO_STOP :
           c8x8g_displayImage( STOP_IMAGE );
       break;
       case DEMO_PLAY :
           c8x8g_displayImage( PLAY_IMAGE );
       break;
       default :

       break;
   }
#endif
}

static void api_playSong()
{
    playStatus = DEMO_PLAY;

#ifdef __SD_CARD__
    while ( 1 != Mmc_Fat_Assign(currentSong, 0));
    Mmc_Fat_Reset(&fileSize);
    bufferStatus = BUFFER_UPLOADED;
#endif
#ifdef __8X8_CLICK__
    c8x8g_displayImage( PLAY_IMAGE );
#endif
}

static void api_stopSong()
{
    playStatus = DEMO_STOP;
    
#ifdef __8X8_CLICK__
    c8x8g_displayImage( STOP_IMAGE );
#endif
}

static void api_nextSong()
{
    if (songStatus == DEMO_CURRENT_SONG)
    {
        songStatus = DEMO_NEXT_SONG;
    }
}

static void api_prevSong()
{
    if (songStatus == DEMO_CURRENT_SONG)
    {
        songStatus = DEMO_PREV_SONG;
    }
}

static void proxfusion_getTouch()
{
    uint8_t touch; 
    
    touch = proxfusion_Touch();
    
    if (touch == 2)
    {
        api_incVolume();
    }
    
    if (touch == 1)
    {
        api_decVolume();
    }
}

static void enocean2_packetHandler( enocean2_packet_t *packet )
{
    DisableInterrupts();
    
    if (packet->type == _ENOCEAN2_TYPE_RADIO_802_15_4)
    {
        if (0 != packet->data_buffer[ _ENOCEAN2_SEQUENCE_NUMBER_OFFSET ])
        {
            switch( packet->data_buffer[ _ENOCEAN2_COMMAND_OFFSET ] )
            {
                case 0x12 :
                {
                    if (muteStatus == DEMO_MUTE)
                    {
                        api_unmuteVolume();
                    }
                    else
                    {
                        api_muteVolume();
                    }
                    Delay_ms( 1000 );
                break;
                }
                case 0x14 :
                {
                    if (playStatus == DEMO_PLAY)
                    {
                        api_stopSong();
                    }
                    else
                    {
                        api_playSong();
                    }
                    Delay_ms( 1000 );
                break;
                }
                case 0x18 :
                {
                    api_nextSong();
                    Delay_ms( 1000 );
                break;
                }
                case 0x22 :
                {
                    api_prevSong();
                    Delay_ms( 1000 );
                break;
                }
                default :
                {
                break;
                }
            }
        }
    }
    EnableInterrupts();
}

////////////////////////////////////////////////////////////// PUBLIC FUNCTIONS

void oled_displaySong(int8_t prevSong, uint8_t song, uint8_t nextSong)
{
#ifdef __OLED_C__
    SPI_Set_Active(SPI2_Read, SPI2_Write);
    OLED_DrawBox(0, 0, 96, 96);
    if (prevSong < 0)prevSong = numMusic;
    if (nextSong > numMusic)nextSong = 0;
    
    OLED_SetFont( guiFont_Tahoma_8_Regular, 0xFFFF, 0 );
    OLED_WriteText(songList[prevSong], 15, 20 );
    OLED_WriteText(songList[nextSong], 15, 70 );
    OLED_SetFont( guiFont_Tahoma_8_Regular, 0xFF00, 0 );
    OLED_WriteText( songList[song] , 15, 45 );
#endif
}

void api_gpioInit()
{

#ifdef __MP3_CLICK__
    _mp3_boardInit();
#endif
#ifdef __OLED_C__
    GPIO_Digital_Output(&PTC_PDOR, _GPIO_PINMASK_13);
    GPIO_Digital_Output(&PTD_PDOR, _GPIO_PINMASK_15);
    GPIO_Digital_Output(&PTE_PDOR, _GPIO_PINMASK_6);
    GPIO_Digital_Output(&PTB_PDOR, _GPIO_PINMASK_20);
#endif
#ifdef __BARGRAPH_CLICK__
    mikrobus_gpioInit( _MIKROBUS1, _MIKROBUS_CS_PIN, _GPIO_OUTPUT );
    mikrobus_gpioInit( _MIKROBUS1, _MIKROBUS_RST_PIN, _GPIO_OUTPUT );
    mikrobus_gpioInit( _MIKROBUS1, _MIKROBUS_PWM_PIN, _GPIO_OUTPUT );
#endif
#ifdef __PROXFUSION_CLICK__
    mikrobus_gpioInit( _MIKROBUS2, _MIKROBUS_INT_PIN, _GPIO_INPUT );
#endif
#ifdef __8X8_CLICK__
    mikrobus_gpioInit( _MIKROBUS3, _MIKROBUS_CS_PIN, _GPIO_OUTPUT );
#endif

}

void api_peripheralInit()
{
// SPI bus is common for all peripherals ( 1MHz, MODE0 )
#if defined (__MP3_CLICK__) || defined (__BARGRAPH_CLICK__) || defined (__8X8_CLICK__)
    mikrobus_spiInit( _MIKROBUS4, &_MP3_SPI_CFG[0] );
#endif
#ifdef __OLED_C__
    SPI2_Init_Advanced(1000000, _SPI_CFG_MASTER | _SPI_CFG_SELECT_CTAR0 | _SPI_CFG_FRAME_SIZE_8BITS |
                       _SPI_CFG_CLK_IDLE_LOW | _SPI_CFG_CLK_PHASE_CAPT_LEADING | _SPI_CFG_MSB_FIRST, &_GPIO_Module_SPI2_PB21_23_22);
    PORTB_PCR21bits.DSE = 1;
    PORTB_PCR22bits.DSE = 1;
    PORTB_PCR23bits.DSE = 1;
#endif
#ifdef __PROXFUSION_CLICK__
    mikrobus_i2cInit( _MIKROBUS2, &_PROXFUSION_I2C_CFG[0] );
#endif
#ifdef __ENOCEAN2_CLICK__
    mikrobus_uartInit( _MIKROBUS4, &_ENOCEAN2_UART_CFG[0] );
#endif
}

void api_driverInit()
{
#ifdef __MP3_CLICK__
    mp3_spiDriverInit( (T_MP3_P)&_MP3_GPIO_OBJ, (T_MP3_P)&_MP3_SPI_OBJ );
#endif
#ifdef __OLED_C__

#endif
#ifdef __BARGRAPH_CLICK__
    bargraph_spiDriverInit( (T_BARGRAPH_P)&_MIKROBUS1_GPIO, (T_BARGRAPH_P)&_MIKROBUS1_SPI );
#endif
#ifdef __PROXFUSION_CLICK__
    proxfusion_i2cDriverInit( (T_PROXFUSION_P)&_MIKROBUS2_GPIO, (T_PROXFUSION_P)&_MIKROBUS2_I2C, 0x44 );
#endif
#ifdef __8X8_CLICK__
    c8x8g_spiDriverInit( (T_C8X8G_P)&_MIKROBUS3_GPIO, (T_C8X8G_P)&_MIKROBUS3_SPI );
#endif
#ifdef __ENOCEAN2_CLICK__
    enocean2_uartDriverInit( (T_ENOCEAN2_P)&_MIKROBUS4_GPIO, (T_ENOCEAN2_P)&_MIKROBUS4_UART );
#endif
}

void api_deviceConfigure()
{
    playStatus = DEMO_STOP;
    songStatus = DEMO_CURRENT_SONG;
    muteStatus = DEMO_UNMUTE;

    bufferStatus = BUFFER_NOT_UPLOADED;
    fileSize = 0;
    songSize = 0;
    currentSongIndex = 0;
    currentVolume = 0x2A2A;
     
    memset( fileBuffer, 0, BUFFER_SIZE);
    memset( dirContent, 0, MAX_SONGLIST_SIZE * 255);
    memset( songList, 0, MAX_SONGLIST_SIZE * MAX_SONGNAME_SIZE);
     
    currentSong = songList[currentSongIndex];
    touchStatus = NOT_TOUCH;

#ifdef __MP3_CLICK__
    mp3_init();
    mp3_reset();
    mp3_cmdWrite(_MP3_MODE_ADDR, 0x0800);
    mp3_cmdWrite(_MP3_BASS_ADDR, 0x7A00);
    mp3_cmdWrite(_MP3_CLOCKF_ADDR, 0x2000);
    mp3_cmdWrite( _MP3_VOL_ADDR, currentVolume );
#endif
#ifdef __OLED_C__
    OLED_Init();
    OLED_setRemap();
    OLED_setCmdFunctions();

    SPI_Set_Active(SPI2_Read, SPI2_Write);
    OLED_SetFont( guiFont_Tahoma_8_Regular, 0xFFFF, 0 );
    OLED_DrawImage( hexiwearLogo_bmp, 0, 0 );
#endif
#ifdef __BARGRAPH_CLICK__
    bargraph_driverInit();
    bargraph_display(10 - ((currentVolume / 256) / 6));
#endif
#ifdef __PROXFUSION_CLICK__
    proxfusion_systemSetting(_PROXFUSION_SYSTEM_RESET |
                             _PROXFUSION_EVENT_MODE_1 |
                             _PROXFUSION_CLOCK_16MHz |
                             _PROXFUSION_ATI_ENABLED |
                             _PROXFUSION_ATI_BAND_1_16 |
                             _PROXFUSION_SYSTEM_ATI_RESET);
    proxfusion_EventMask( _PROXFUSION_MASK_SYS_EVENT );
#endif
#ifdef __8X8_CLICK__
    c8x8g_writeCmd(_C8X8G_DECODE_MODE_REG, _C8X8G_NO_DECODE);
    c8x8g_writeCmd(_C8X8G_INTENSITY_REG,   _C8X8G_INTENSITY_15);
    c8x8g_writeCmd(_C8X8G_SCAN_LIMIT_REG,  _C8X8G_DISPLAY_DIGIT_0_7);
    c8x8g_writeCmd(_C8X8G_SHUTDOWN_REG,    _C8X8G_NORMAL_OPERATION);
    c8x8g_setSpeedScroll(_C8X8G_SPEED_MEDIUM);
    c8x8g_displayRefresh();
#endif
#ifdef __ENOCEAN2_CLICK__
    enocean2_init(&enocean_rxBuffer[0], _ENOCEAN2_RX_BUFFER_SIZE, &enocean_dataBuffer[0], _ENOCEAN2_RX_BUFFER_SIZE );
    enocean2_setCallbackHandler(enocean2_packetHandler);
    UART3_C2 |= 1 << 5;
    NVIC_IntEnable(IVT_INT_UART3_RX_TX);
    EnableInterrupts();
#endif
#ifdef __HEXIWEAR_TOUCH__
    UART4_Init_Advanced( 230400, _UART_8_BIT_DATA, _UART_NOPARITY,
                        _UART_TWO_STOPBITS, &_GPIO_Module_UART4_PE24_25 );

    UART4_C2 |= 1 << 5;
    NVIC_IntEnable( IVT_INT_UART4_RX_TX );
    EnableInterrupts();
#endif
    
    InitTimer0();
    InitTimer1();
    api_getSongList();
    api_playSong();
    
    Delay_ms( 3000 );
#ifdef __OLED_C__
    oled_displaySong(currentSongIndex - 1, currentSongIndex, currentSongIndex + 1);
#endif
}

void api_process()
{
    uint8_t c;
    uint8_t t;
    
#ifdef __ENOCEAN2_CLICK__
    enocean2_process();
#endif
    
    if (playStatus == DEMO_PLAY)
    {
        if (fileSize > BUFFER_SIZE)
        {
#ifdef __MP3_CLICK__
            if (!mp3_dataWrite32( fileBuffer ))
            {
                bufferStatus = BUFFER_UPLOADED;
            }
            else
            {
                bufferStatus = BUFFER_NOT_UPLOADED;
            }
#endif
            if (bufferStatus == BUFFER_UPLOADED)
            {
                for (c = 0; c < 32; c++)
                {
#ifdef __SD_CARD__
                    Mmc_Fat_Read( fileBuffer + c );
#endif
                }
                fileSize -= BUFFER_SIZE;
            }
        }
        else
        {
            if (bufferStatus == BUFFER_UPLOADED)
            {

                for (t = 0; t < fileSize; t++)
                {
#ifdef __SD_CARD__
                    Mmc_Fat_Read( fileBuffer[t] );
#endif
                }
                fileSize = 0;
            }
#ifdef __MP3_CLICK__
            if (!mp3_dataWrite32( fileBuffer ))
            {
                bufferStatus = BUFFER_UPLOADED;
            }
            else
            {
                bufferStatus = BUFFER_NOT_UPLOADED;
            }
#endif
        }
    }

    if (songStatus == DEMO_NEXT_SONG)
    {
        currentSongIndex++;
        if (currentSongIndex > numMusic)
        {
            currentSongIndex = 0;
        }
        
        currentSong = songList[ currentSongIndex ];
        Delay_ms( 2000 );
#ifdef __OLED_C__
        oled_displaySong(currentSongIndex - 1 , currentSongIndex, currentSongIndex + 1);
#endif
        api_playSong();

        songStatus = DEMO_CURRENT_SONG;
    }
    
    if (songStatus == DEMO_PREV_SONG)
    {
        currentSongIndex--;
        if (currentSongIndex < 0)
        {
            currentSongIndex = numMusic;
        }

        currentSong = songList[ currentSongIndex ];
        Delay_ms( 2000 );
#ifdef __OLED_C__
        oled_displaySong(currentSongIndex - 1 , currentSongIndex, currentSongIndex + 1);
#endif
        api_playSong();
        
        songStatus = DEMO_CURRENT_SONG;
    }
}

/////////////////////////////////////////////////////////////// TIMER INTERRUPT

void Timer0_interrupt() iv IVT_INT_PIT0
{
   PIT_TFLG0.TIF = 1;
#ifdef __PROXFUSION_CLICK__
    proxfusion_getTouch();
#endif
}


void Timer1_interrupt() iv IVT_INT_PIT1
{
    PIT_TFLG1.TIF = 1;
#ifdef __HEXIWEAR_TOUCH__
    if (touchStatus == NEW_TOUCH && touchBuffer[2] >= 0x00 && touchBuffer[2] <= 0x03)
    {
        switch( touchBuffer[2] )
        {
            case 0x03 :
            {
                if (muteStatus == DEMO_MUTE)
                {
                    api_unmuteVolume();
                }
                else
                {
                    api_muteVolume();
                }
                Delay_ms( 1000 );
            break;
            }
            case 0x02 :
            {
                if (playStatus == DEMO_PLAY)
                {
                    api_stopSong();
                }
                else
                {
                    api_playSong();
                }
                Delay_ms( 1000 );
            break;
            }
            case 0x00 :
            {
                api_nextSong();
                Delay_ms( 1000 );
            break;
            }
            case 0x01 :
            {
                api_prevSong();
                Delay_ms( 1000 );
            break;
            }
            default :
            {
            break;
            }
        }
        tBuferCounter = 0;
        touchStatus = NOT_TOUCH;
    }
#endif
}
//////////////////////////////////////////////////////////////// UART INTERRUPT

void MKW40_RX_ISR() iv IVT_INT_UART4_RX_TX ics ICS_AUTO
{
    char s1 = UART4_S1;
    uint8_t tmp = UART4_D;

    if(  s1 & ( 1 << 5 )  )
    {
#ifdef __HEXIWEAR_TOUCH__
        touchBuffer[tBuferCounter++] = tmp;
        touchStatus = NEW_TOUCH;
#endif
    }

}

void ENOCEAN_RX_ISR() iv IVT_INT_UART3_RX_TX ics ICS_AUTO
{
    if( UART3_S1 & ( 1 << 5 )  )
    {
        char tmp = UART3_D;
#ifdef __ENOCEAN2_CLICK__
        enocean2_rx( tmp );
#endif
    }
}