#include "OLED_driver.h"
#include "OLED_info.h"

// Serial Flash Chip Select connection
sbit OLED_Pwr_bit           at PTC_PDOR.B13;
sbit OLED_DC_bit            at PTD_PDOR.B15;
sbit OLED_RST_bit           at PTE_PDOR.B6;
sbit OLED_CS_bit            at PTB_PDOR.B20;

static uint8_t isFontInitialized = 0;
static uint8_t* selectedFont;

static uint16_t  selectedFont_color;
static uint16_t  selectedFont_firstChar;
static uint16_t  selectedFont_lastChar;
static uint16_t  selectedFont_height;

static uint8_t screenBuf[ OLED_GRAM_SIZE ];
static uint8_t  currentChar_width  = 0;
static uint8_t  currentChar_height = 0;

static uint16_t FontColor;
static uint8_t FontOrientation;
const uint8_t FO_HORIZONTAL = 0;
const uint8_t FO_VERTICAL = 1;
const uint8_t FO_VERTICAL_COLUMN = 2;
char ExternalFontSet;
static uint8_t FontInitialized = 0;

static uint8_t* _font;
static uint16_t _fontFirstChar;
static uint16_t _fontLastChar;
static uint16_t _fontHeight;
static uint16_t x_cord;
static uint16_t y_cord;

static uint8_t OLED_REMAP_SETTINGS = REMAP_ORDER_ABC | REMAP_COM_SPLIT_ODD_EVEN_EN |
                                     REMAP_COLOR_RGB565 | REMAP_COLUMNS_LEFT_TO_RIGHT |
                                     REMAP_SCAN_UP_TO_DOWN | REMAP_HORIZONTAL_INCREMENT;
                                     

static void SetBorders(uint8_t xCrd, uint8_t yCrd, uint8_t width, uint8_t height);
static void OLED_Swap(uint16_t* imgDst, const uint8_t* imgSrc, uint16_t imgSize);

static void OLED_WriteChar(uint16_t ch);
static void OLED_MoveCursor(uint16_t _x, uint16_t _y);

void OLED_PowerOn()
{
    OLED_Pwr_bit = 1;
}

void OLED_PowerOff()
{
    OLED_Pwr_bit = 0;
}

uint8_t OLED_SendCmd (uint32_t cmd, uint8_t isFirst)
{
    if ( isFirst )
    {
        OLED_DC_bit = 0;
    }
    else
    {
        OLED_DC_bit = 1;
    }
    OLED_CS_bit = 0;
    SPI2_Write(cmd);
    OLED_CS_bit = 1;
    return OLED_STATUS_SUCCESS;
}

 
uint8_t OLED_Init()
{
    OLED_PowerOff();
    Delay_1ms();
    OLED_RST_bit = 0;
    Delay_1ms();
    OLED_RST_bit = 1;
    Delay_1ms();
    OLED_PowerOn();

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_SET_CMD_LOCK, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_UNLOCK, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_SET_CMD_LOCK, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_ACC_TO_CMD_YES, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_DISPLAYOFF, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_SET_OSC_FREQ_AND_CLOCKDIV, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0xF1, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_SET_MUX_RATIO, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0x5F, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
}
uint8_t OLED_setRemap()
{
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_SET_REMAP, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_REMAP_SETTINGS, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_SET_COLUMN, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0x00, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0x5F, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_SET_ROW, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0x00, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0x5F, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_STARTLINE, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0x80, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_DISPLAYOFFSET, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0x60, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_PRECHARGE, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0x32, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_VCOMH, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0x05, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_NORMALDISPLAY, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
}
uint8_t OLED_setCmdFunctions()
{

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_CONTRASTABC, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0x8A, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0x51, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0x8A, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_CONTRASTMASTER, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0xCF, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_SETVSL, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0xA0, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0xB5, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0x55, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_PRECHARGE2, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( 0x01, OTHER_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_DISPLAYON, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }

    return OLED_STATUS_SUCCESS;
}

uint8_t OLED_SendData(const uint8_t* dataToSend,uint32_t dataSize)
{
    uint8_t spiStatus;
    const uint8_t* bufPtr;

    if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_WRITERAM, FIRST_BYTE ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }
    OLED_DC_bit = 1;
    OLED_CS_bit = 0;

    bufPtr = dataToSend;

    while(dataSize--)
    {
        SPI2_Write(*(bufPtr++));
    }

    OLED_CS_bit = 1;
    return OLED_STATUS_SUCCESS;
}

uint8_t OLED_DrawBox ( uint16_t xCrd, uint16_t yCrd, uint16_t width, uint16_t height )
{
    uint8_t status = OLED_STATUS_SUCCESS;
    uint32_t boxSize;
    uint16_t color = 0x0000;

    boxSize = width*height;

    // check the bounds
    if (0 != ((( xCrd + width - 1 ) > OLED_SCREEN_WIDTH ) || ( xCrd < 0 ) || ( ( yCrd + height - 1 ) > OLED_SCREEN_HEIGHT ) || ( yCrd < 0 )))
    {
        status = OLED_STATUS_INIT_ERROR;
    }
    else
    {
        uint16_t cnt;
        xCrd += OLED_COLUMN_OFFSET;
        yCrd += OLED_ROW_OFFSET;

        if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_SET_COLUMN, FIRST_BYTE ) )
        {
            return OLED_STATUS_PROTOCOL_ERROR;
        }
        if ( OLED_STATUS_SUCCESS != OLED_SendCmd( xCrd, OTHER_BYTE ) )
        {
            return OLED_STATUS_PROTOCOL_ERROR;
        }
        if ( OLED_STATUS_SUCCESS != OLED_SendCmd( xCrd + (width-1), OTHER_BYTE ) )
        {
            return OLED_STATUS_PROTOCOL_ERROR;
        }

        if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_SET_ROW, FIRST_BYTE ) )
        {
            return OLED_STATUS_PROTOCOL_ERROR;
        }
        if ( OLED_STATUS_SUCCESS != OLED_SendCmd( yCrd, OTHER_BYTE ) )
        {
            return OLED_STATUS_PROTOCOL_ERROR;
        }
        if ( OLED_STATUS_SUCCESS != OLED_SendCmd( yCrd + (height-1), OTHER_BYTE ) )
        {
            return OLED_STATUS_PROTOCOL_ERROR;
        }

        if ( OLED_STATUS_SUCCESS != OLED_SendCmd( OLED_CMD_WRITERAM, FIRST_BYTE ) )
        {
            return OLED_STATUS_PROTOCOL_ERROR;
        }

        OLED_DC_bit = 1;
        OLED_CS_bit = 0;
        for(cnt = 0; cnt < boxSize; cnt++)
        {
            SPI2_Write(color >> 8);
            SPI2_Write(color);
        }
        OLED_CS_bit = 1;
        
        status = OLED_STATUS_SUCCESS;
    }

  return status;
}
uint8_t OLED_FillScreen( uint16_t color )
{
    return OLED_DrawBox(0, 0, 96, 96);
}

uint8_t OLED_DrawPixel (int16_t xCrd, int16_t yCrd, uint16_t color)
{

   if (0 != ( xCrd > OLED_SCREEN_WIDTH ))
    {
        return OLED_STATUS_INIT_ERROR;
    }
    if (0 != ( xCrd < 0 ))
    {
        return OLED_STATUS_INIT_ERROR;
    }
    if (0 != ( yCrd  > OLED_SCREEN_HEIGHT ))
    {
        return OLED_STATUS_INIT_ERROR;
    }
    if (0 != ( yCrd < 0 ))
    {
        return OLED_STATUS_INIT_ERROR;
    }
    else
    {
        uint16_t dot = color;
        SetBorders( xCrd, yCrd, OLED_SCREEN_WIDTH, OLED_SCREEN_HEIGHT);

        dot = (( dot & 0xFF00 ) >> 8 ) | ( ( dot & 0x00FF ) << 8 );

        if ( OLED_STATUS_SUCCESS != OLED_SendData( &dot, 2 ) )
        {
            return OLED_STATUS_PROTOCOL_ERROR;
        }
        return OLED_STATUS_SUCCESS;
    }
}

static void SetBorders(uint8_t xCrd, uint8_t yCrd, uint8_t width, uint8_t height)
{
    uint8_t oledStatus;
    xCrd += OLED_COLUMN_OFFSET;
    yCrd += OLED_ROW_OFFSET;

    while (1)
    {
        oledStatus = OLED_STATUS_SUCCESS;

        oledStatus |= OLED_SendCmd( OLED_CMD_SET_COLUMN, FIRST_BYTE );
        oledStatus |= OLED_SendCmd( xCrd, OTHER_BYTE );
        oledStatus |= OLED_SendCmd( xCrd + (width-1), OTHER_BYTE );
        oledStatus |= OLED_SendCmd( OLED_CMD_SET_ROW, FIRST_BYTE );
        oledStatus |= OLED_SendCmd( yCrd, OTHER_BYTE );
        oledStatus |= OLED_SendCmd( yCrd + (height-1), OTHER_BYTE );

        if ( OLED_STATUS_SUCCESS == oledStatus)
        {
            break;
        }
    }
}


static void OLED_Swap(uint16_t* imgDst,const uint8_t* imgSrc,uint16_t imgSize)
{
    uint32_t var;
    for (var = 0; var < imgSize; var++)
    {
        *imgDst = *imgSrc << 8;
        imgSrc++;
        *imgDst |= *imgSrc;
        imgDst++;
        imgSrc++;
    }
}


void OLED_GetImageDimensions(uint8_t *width, uint8_t *height, const uint8_t* image)
{
  *height = image[2] + (image[3] << 8);
  *width  = image[4] + (image[5] << 8);
}


uint8_t OLED_DrawImage (const uint8_t* image, uint8_t xCrd, uint8_t yCrd)
{
    uint8_t status = OLED_STATUS_SUCCESS;
    uint8_t *ptr;
    uint16_t width;
    uint16_t height;
    
    ptr = image;
    
    ptr += 2;
    height = *ptr;
    ptr++;
    height += (*ptr) << 8;

    ptr++;
    width = *ptr;
    ptr++;
    width += (*ptr)<<8;

    if (0 != ((( xCrd + width - 1 ) > OLED_SCREEN_WIDTH ) || ( xCrd < 0 ) || ( ( yCrd + height - 1 ) > OLED_SCREEN_HEIGHT ) || ( yCrd < 0 )))
    {
        status = OLED_STATUS_INIT_ERROR;
    }
    else
    {
        OLED_Swap( screenBuf,  image + 6 , width * height );
    }

    SetBorders( xCrd, yCrd, width, height );
    if ( OLED_STATUS_SUCCESS != OLED_SendData( screenBuf, width * height * OLED_BYTES_PER_PIXEL ) )
    {
        return OLED_STATUS_PROTOCOL_ERROR;
    }

    return OLED_STATUS_SUCCESS;
}

void OLED_SetFont(const uint8_t *activeFont, uint16_t font_color, uint8_t font_orientation)
{
    _font = activeFont;
    _fontFirstChar = activeFont[2] + (activeFont[3] << 8);
    _fontLastChar = activeFont[4] + (activeFont[5] << 8);
    _fontHeight = activeFont[6];

    FontColor = font_color;
    FontOrientation = font_orientation;
    FontInitialized = 1;
    ExternalFontSet = 0;
}

void OLED_WriteText(uint8_t *text, uint16_t x, uint16_t y) 
{
  uint16_t i;

  if (x >= OLED_SCREEN_WIDTH)
  {
    return;
  }
  if (y >= OLED_SCREEN_HEIGHT)
  {
    return;
  }

  i = 0;
  OLED_MoveCursor(x, y);
  while (text[i])
  {
      OLED_WriteChar(text[i++]);
  }
}
static void OLED_MoveCursor(uint16_t _x, uint16_t _y)
{
    x_cord = _x;
    y_cord = _y;
}

static void OLED_WriteChar(uint16_t ch)
{
    uint8_t chWidth = 0;
    uint8_t xCnt;
    uint8_t yCnt;
    uint16_t x = 0;
    uint16_t y;
    uint8_t temp = 0;
    uint8_t mask;
    uint32_t offset;
    const uint8_t *pChTable;
    const uint8_t *pChBitMap;
    uint16_t tmp;

    if(FontInitialized == 0)
    {
        OLED_SetFont(guiFont_Tahoma_8_Regular, OLED_COLOR_BLACK, FO_HORIZONTAL); //default font
        FontInitialized = 1;
    }

    if (ch < _fontFirstChar)
    {
        return;
    }
    if (ch > _fontLastChar)
    {
        return;
    }

    offset = 0;
    tmp = (ch - _fontFirstChar) << 2;
    pChTable = _font + 8 + tmp;
    chWidth = *pChTable;

    offset = (uint32_t) pChTable[1] + ((uint32_t)pChTable[2] << 8) + ((uint32_t)pChTable[3] << 16);
    pChBitMap = _font + offset;

    if (((FontOrientation == FO_HORIZONTAL) || (FontOrientation == FO_VERTICAL_COLUMN)) != 0)
    {
        y = y_cord;
        for (yCnt = 0; yCnt < _fontHeight; yCnt++) 
        {
            x = x_cord;
            mask = 0;
            for (xCnt = 0; xCnt < chWidth; xCnt++) 
            {
                if (mask == 0) 
                {
                    temp = *(pChBitMap++);
                    mask = 0x01;
                }

                if ((temp & mask) != 0)
                {
                    OLED_DrawPixel(x, y, FontColor);
                }

                x++;
                mask <<= 1;
            }
            y++;
        }
        if (FontOrientation == FO_HORIZONTAL)
        {
            x_cord = x + 1;
        }
        else
        {
            y_cord = y;
        }
    }
    else 
    {
        y = x_cord;
        for (yCnt = 0; yCnt < _fontHeight; yCnt++) 
        {
            x = y_cord;
            mask = 0;
            for (xCnt = 0; xCnt < chWidth; xCnt++) 
            {
                if (mask == 0) 
                {
                    temp = *(pChBitMap++);
                    mask = 0x01;
                }

                if (temp & mask) 
                {
                    OLED_DrawPixel(y, x, FontColor);
                }

                x--;
                mask <<= 1;
            }

            y++;
        }
        y_cord = x - 1;
    }
}