// command byte number
const uint8_t FIRST_BYTE = 1;
const uint8_t OTHER_BYTE = 0;

const uint8_t OLED_COLUMN_OFFSET = 16;
const uint8_t OLED_ROW_OFFSET = 0;
const uint8_t OLED_SCREEN_WIDTH = 96;
const uint8_t OLED_SCREEN_HEIGHT = 96;

const uint8_t OLED_SCREEN_WIDTH_END  =  ( OLED_SCREEN_WIDTH - 1 )  + OLED_COLUMN_OFFSET;
const uint8_t OLED_SCREEN_HEIGHT_END =  ( OLED_SCREEN_HEIGHT - 1 ) + OLED_ROW_OFFSET;

const uint8_t OLED_BYTES_PER_PIXEL = 2;
const uint8_t OLED_TRANSITION_STEP = 1;
const uint8_t OLED_ACTION_DCON = 0xFE;
const uint8_t OLED_ACTION_DCOFF = 0xFF;

const uint8_t OLED_CHAR_WIDTH_AUTO = 0xFF;
const uint8_t OLED_CHAR_WIDTH_MAX = 0xFE;
const uint8_t OLED_CHAR_WIDTH_NUM = 0xFD;
const uint8_t OLED_CHAR_WIDTH_KEEP = 0xFC;
const uint8_t OLED_CHAR_HEIGHT_AUTO = 0xFB;
const uint8_t OLED_CHAR_HEIGHT_KEEP = 0xFA;

const uint16_t OLED_SCREEN_SIZE = OLED_SCREEN_WIDTH * OLED_SCREEN_HEIGHT;
const uint16_t OLED_GRAM_SIZE = OLED_SCREEN_WIDTH * OLED_SCREEN_HEIGHT * OLED_BYTES_PER_PIXEL;

const uint8_t OLED_TEXT_HALIGN_SHIFT = 0;
const uint8_t OLED_TEXT_HALIGN_MASK = 0x03 << OLED_TEXT_HALIGN_SHIFT;
const uint8_t OLED_TEXT_VALIGN_SHIFT = 4;
const uint8_t OLED_TEXT_VALIGN_MASK = 0x03 << OLED_TEXT_VALIGN_SHIFT;

// REMAP settings
const uint8_t REMAP_HORIZONTAL_INCREMENT  = 0;
const uint8_t REMAP_VERTICAL_INCREMENT    = 1;
const uint8_t REMAP_COLUMNS_LEFT_TO_RIGHT = 0;
const uint8_t REMAP_COLUMNS_RIGHT_TO_LEFT = 1 << 1;
const uint8_t REMAP_ORDER_ABC = 0;
const uint8_t REMAP_ORDER_CBA = 1 << 2;
const uint8_t REMAP_SCAN_UP_TO_DOWN = 0;
const uint8_t REMAP_SCAN_DOWN_TO_UP = 1 << 4;
const uint8_t REMAP_COM_SPLIT_ODD_EVEN_DIS = 0;
const uint8_t REMAP_COM_SPLIT_ODD_EVEN_EN = 1 << 5;
const uint8_t REMAP_COLOR_RGB565 = 1 << 6;

//the 2nd and 3rd byte represent the start and the end address, respectively
const uint8_t OLED_CMD_SET_COLUMN = 0x15;
const uint8_t OLED_CMD_SET_ROW = 0x75;

const uint8_t OLED_DIRECTION_HORIZONTAL = 0;
const uint8_t OLED_DIRECTION_VERTICAL = 1;

// SET DISPLAY
const uint8_t OLED_CMD_SET_DISPLAY_MODE_ALL_OFF = 0xA4;
const uint8_t OLED_CMD_SET_DISPLAY_MODE_ALL_ON = 0xA5;
const uint8_t OLED_CMD_SET_DISPLAY_MODE_NORMAL = 0xA6;
const uint8_t OLED_CMD_SET_DISPLAY_MODE_INVERSE = 0xA7;

const uint8_t OLED_CMD_SET_CMD_LOCK = 0xFD;
const uint8_t OLED_UNLOCK = 0x12;
const uint8_t OLED_LOCK = 0x16;
const uint8_t OLED_ACC_TO_CMD_NO = 0xB0;
const uint8_t OLED_ACC_TO_CMD_YES = 0xB1;

const uint8_t OLED_CMD_NOP = 0xD1; // also, 0xE3
const uint8_t OLED_CMD_SET_MUX_RATIO = 0xCA;

const uint8_t OLED_CMD_SET_REMAP = 0xA0;

// set horisontal or vertical increment
const uint8_t OLED_ADDR_INC_HOR = 0x00;
const uint8_t OLED_ADDR_INC_VER = 0x01;
const uint8_t OLED_COLUMN_ADDR_REMAP_0_TO_SEG0 = 0x00;
const uint8_t OLED_COLUMN_ADDR_REMAP_127_TO_SEG0 = 0x02;

// color sequence order
const uint8_t OLED_COLOR_SEQ_A_B_C = 0x00;
const uint8_t OLED_COLOR_SEQ_C_B_A = 0x04;

// scanning order (MR == MUX ratio)
const uint8_t OLED_SCAN_FROM_COM_0_TO_MR = 0x00;
const uint8_t OLED_SCAN_FROM_COM_MR_TO_0 = 0x10;

// COM splitting to odd and even
const uint8_t OLED_COM_SPLIT_DISABLE = 0x00;
const uint8_t OLED_COM_SPLIT_ENABLE = 0x20;

// screen color depth
const uint8_t OLED_COLOR_DEPTH_256    = 0x00;
const uint8_t OLED_COLOR_DEPTH_65K    = 0x40;
const uint8_t OLED_COLOR_DEPTH_262K_1 = 0x80;
const uint8_t OLED_COLOR_DEPTH_262K_2 = 0xC0;

const uint8_t OLED_CMD_SET_RESET_AND_PRECHARGE_PERIOD = 0xB1;

const uint8_t OLED_RESET_PERIOD_5  = 0x02;
const uint8_t OLED_RESET_PERIOD_7  = 0x03;
const uint8_t OLED_RESET_PERIOD_9  = 0x04;
const uint8_t OLED_RESET_PERIOD_11 = 0x05;
const uint8_t OLED_RESET_PERIOD_13 = 0x06;
const uint8_t OLED_RESET_PERIOD_15 = 0x07;
const uint8_t OLED_RESET_PERIOD_17 = 0x08;
const uint8_t OLED_RESET_PERIOD_19 = 0x09;
const uint8_t OLED_RESET_PERIOD_21 = 0x0A;
const uint8_t OLED_RESET_PERIOD_23 = 0x0B;
const uint8_t OLED_RESET_PERIOD_25 = 0x0C;
const uint8_t OLED_RESET_PERIOD_27 = 0x0D;
const uint8_t OLED_RESET_PERIOD_29 = 0x0E;
const uint8_t OLED_RESET_PERIOD_31 = 0x0F;

const uint8_t OLED_PRECHARGE_PERIOD_3  = 0x03;
const uint8_t OLED_PRECHARGE_PERIOD_4  = 0x04;
const uint8_t OLED_PRECHARGE_PERIOD_5  = 0x05;
const uint8_t OLED_PRECHARGE_PERIOD_6  = 0x06;
const uint8_t OLED_PRECHARGE_PERIOD_7  = 0x07;
const uint8_t OLED_PRECHARGE_PERIOD_8  = 0x08;
const uint8_t OLED_PRECHARGE_PERIOD_9  = 0x09;
const uint8_t OLED_PRECHARGE_PERIOD_10 = 0x0A;
const uint8_t OLED_PRECHARGE_PERIOD_11 = 0x0B;
const uint8_t OLED_PRECHARGE_PERIOD_12 = 0x0C;
const uint8_t OLED_PRECHARGE_PERIOD_13 = 0x0D;
const uint8_t OLED_PRECHARGE_PERIOD_14 = 0x0E;
const uint8_t OLED_PRECHARGE_PERIOD_15 = 0x0F;

const uint8_t OLED_CMD_SET_OSC_FREQ_AND_CLOCKDIV = 0xB3;

// clock divider
const uint8_t OLED_CLOCKDIV_1    = 0x00;
const uint8_t OLED_CLOCKDIV_2    = 0x01;
const uint8_t OLED_CLOCKDIV_4    = 0x02;
const uint8_t OLED_CLOCKDIV_8    = 0x03;
const uint8_t OLED_CLOCKDIV_16   = 0x04;
const uint8_t OLED_CLOCKDIV_32   = 0x05;
const uint8_t OLED_CLOCKDIV_64   = 0x06;
const uint8_t OLED_CLOCKDIV_128  = 0x07;
const uint8_t OLED_CLOCKDIV_256  = 0x08;
const uint8_t OLED_CLOCKDIV_512  = 0x09;
const uint8_t OLED_CLOCKDIV_1024 = 0x0A;

const uint8_t OLED_OSC_FREQ = 0xB0;
const uint8_t OLED_CMD_STARTLINE  = 0xA1;

const uint8_t OLED_CMD_WRITERAM       = 0x5C;
const uint8_t OLED_CMD_READRAM        = 0x5D;
const uint8_t OLED_CMD_DISPLAYOFFSET  = 0xA2;
const uint8_t OLED_CMD_DISPLAYALLOFF  = 0xA4;
const uint8_t OLED_CMD_DISPLAYALLON   = 0xA5;
const uint8_t OLED_CMD_NORMALDISPLAY  = 0xA6;
const uint8_t OLED_CMD_INVERTDISPLAY  = 0xA7;
const uint8_t OLED_CMD_FUNCTIONSELECT = 0xAB;
const uint8_t OLED_CMD_DISPLAYOFF     = 0xAE;
const uint8_t OLED_CMD_DISPLAYON      = 0xAF;
const uint8_t OLED_CMD_PRECHARGE      = 0xB1;
const uint8_t OLED_CMD_DISPLAYENHANCE = 0xB2;
const uint8_t OLED_CMD_SETVSL         = 0xB4;
const uint8_t OLED_CMD_SETGPIO        = 0xB5;
const uint8_t OLED_CMD_PRECHARGE2     = 0xB6;
const uint8_t OLED_CMD_SETGRAY        = 0xB8;
const uint8_t OLED_CMD_USELUT         = 0xB9;
const uint8_t OLED_CMD_PRECHARGELEVEL = 0xBB;
const uint8_t OLED_CMD_VCOMH          = 0xBE;
const uint8_t OLED_CMD_CONTRASTABC    = 0xC1;
const uint8_t OLED_CMD_CONTRASTMASTER = 0xC7;
const uint8_t OLED_CMD_MUXRATIO       = 0xCA;
const uint8_t OLED_CMD_COMMANDLOCK    = 0xFD;
const uint8_t OLED_CMD_HORIZSCROLL    = 0x96;
const uint8_t OLED_CMD_STOPSCROLL     = 0x9E;
const uint8_t OLED_CMD_STARTSCROLL    = 0x9F;

// STATUS
const uint8_t OLED_STATUS_SUCCESS = 0;        // success
const uint8_t OLED_STATUS_ERROR = 1;          // fail
const uint8_t OLED_STATUS_PROTOCOL_ERROR = 2; // SPI failure
const uint8_t OLED_STATUS_INIT_ERROR = 3;     // initialization error
const uint8_t OLED_STATUS_DEINIT_ERROR = 4;    // deinitialization error

// COLOR
const uint16_t OLED_COLOR_BLACK    = 0x0000;
const uint16_t OLED_COLOR_BLUE_1   = 0x06FF;
const uint16_t OLED_COLOR_BLUE     = 0x001F;
const uint16_t OLED_COLOR_RED      = 0xF800;
const uint16_t OLED_COLOR_GREEN    = 0x07E0;
const uint16_t OLED_COLOR_CYAN     = 0x07FF;
const uint16_t OLED_COLOR_MAGENTA  = 0xF81F;
const uint16_t OLED_COLOR_YELLOW   = 0xFFE0;
const uint16_t OLED_COLOR_GRAY     = 0x528A;
const uint16_t OLED_COLOR_WHITE    = 0xFFFF;

// TEXT ALIGN
const uint8_t OLED_TEXT_ALIGN_NONE = 0;
const uint8_t OLED_TEXT_ALIGN_LEFT   = 0x1;
const uint8_t OLED_TEXT_ALIGN_RIGHT  = 0x2;
const uint8_t OLED_TEXT_ALIGN_CENTER = 0x3;
const uint8_t OLED_TEXT_VALIGN_TOP    = 0x10;
const uint8_t OLED_TEXT_VALIGN_BOTTOM = 0x20;
const uint8_t OLED_TEXT_VALIGN_CENTER = 0x30;

// OLED TRANSITIIN
const uint8_t OLED_TRANSITION_NONE = 0;
const uint8_t OLED_TRANSITION_TOP_DOWN = 1;
const uint8_t OLED_TRANSITION_DOWN_TOP = 2;
const uint8_t OLED_TRANSITION_LEFT_RIGHT = 3;
const uint8_t OLED_TRANSITION_RIGHT_LEFT = 4;
